=== modified file 'bzrlib/changeset.py'
--- bzrlib/changeset.py	2005-12-22 23:32:43 +0000
+++ bzrlib/changeset.py	2006-01-13 05:52:00 +0000
@@ -1178,29 +1178,6 @@
     return changed_inventory
 
 
-def apply_changeset_tree(cset, tree):
-    r_inventory = {}
-    for entry in tree.source_inventory().itervalues():
-        inventory[entry.id] = entry.path
-    new_inventory = apply_changeset(cset, r_inventory, tree.basedir)
-    new_entries, remove_entries = \
-        get_inventory_change(inventory, new_inventory, cset)
-    tree.update_source_inventory(new_entries, remove_entries)
-
-
-def get_inventory_change(inventory, new_inventory, cset):
-    new_entries = {}
-    remove_entries = []
-    for entry in cset.entries.itervalues():
-        if entry.needs_rename():
-            new_path = entry.get_new_path(inventory, cset)
-            if new_path is None:
-                remove_entries.append(entry.id)
-            else:
-                new_entries[new_path] = entry.id
-    return new_entries, remove_entries
-
-
 def print_changeset(cset):
     """Print all non-boring changeset entries
     

=== modified file 'bzrlib/hashcache.py'
--- bzrlib/hashcache.py	2005-12-01 17:36:29 +0000
+++ bzrlib/hashcache.py	2006-01-13 05:52:00 +0000
@@ -32,12 +32,14 @@
 import os, stat, time
 import sha
 
-from bzrlib.osutils import sha_file
+from bzrlib.osutils import sha_file, pathjoin
 from bzrlib.trace import mutter, warning
 from bzrlib.atomicfile import AtomicFile
-from bzrlib.osutils import pathjoin
-
-
+from bzrlib.errors import BzrError
+
+
+FP_MTIME_COLUMN = 1
+FP_CTIME_COLUMN = 2
 FP_MODE_COLUMN = 5
 
 def _fingerprint(abspath):
@@ -170,16 +172,23 @@
         if stat.S_ISREG(mode):
             digest = sha_file(file(abspath, 'rb', buffering=65000))
         elif stat.S_ISLNK(mode):
-            link_target = os.readlink(abspath)
             digest = sha.new(os.readlink(abspath)).hexdigest()
         else:
             raise BzrError("file %r: unknown file stat mode: %o"%(abspath,mode))
 
         now = int(time.time())
-        if file_fp[1] >= now or file_fp[2] >= now:
+        if file_fp[FP_MTIME_COLUMN] >= now or file_fp[FP_CTIME_COLUMN] >= now:
             # changed too recently; can't be cached.  we can
             # return the result and it could possibly be cached
             # next time.
+            #
+            # the point is that we only want to cache when we are sure that any
+            # subsequent modifications of the file can be detected.  If a
+            # modification neither changes the inode, the device, the size, nor
+            # the mode, then we can only distinguish it by time; therefore we
+            # need to let sufficient time elapse before we may cache this entry
+            # again.  If we didn't do this, then, for example, a very quick 1
+            # byte replacement in the file might go undetected.
             self.danger_count += 1 
             if cache_fp:
                 self.removed_count += 1

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-01-02 22:54:45 +0000
+++ bzrlib/workingtree.py	2006-01-13 05:52:00 +0000
@@ -914,7 +914,7 @@
         between multiple working trees, i.e. via shared storage, then we 
         would probably want to lock both the local tree, and the branch.
         """
-        if self._hashcache.needs_write:
+        if self._hashcache.needs_write and self.branch._lock_count==1:
             self._hashcache.write()
         return self.branch.unlock()
 

